
/* PAIRLIST 1.2 */
/* Questa libreria permette di definire e gestire una lista di coppie
   (nome, valore) allocata dinamicamente. Vedi piu' sotto per l'uso. */
   

#ifndef _PAIRLIST

#define _PAIRLIST


#include <stdlib.h>
#include <string.h>

#define MAX_NAME 15
#define MAX_VALUE 61

typedef int T_LIST_VAL;
typedef struct SP *TP_PAIR;
typedef struct SP {
	   char Name[MAX_NAME];
	   char Value[MAX_VALUE];
	   TP_PAIR Next;
		  } T_PAIR;
typedef struct {
    TP_PAIR Head;
    int NElem;
    TP_PAIR Current;
	       } T_LIST;


extern int ListCreate(T_LIST *);
extern void ListFree(T_LIST *);
extern char *ListRead(T_LIST *, char *);
extern int ListPutFront(T_LIST *, char *, char *);
extern int ListIsEmpty(T_LIST *);

/* ListCreate crea il dummy element (il descrittore deve essere definito
   esternamente, ma puo' anche non essere stato inizializzato). Restituisce
   1 se ok, 0 se non puo' creare.
   ListFree elimina tutti gli elementi della lista (non e' necessario svuo-
   tarla in precedenza) incluso il dummy element. Dopo di averla chiamata
   si puo' uscire dal programma o ricreare la lista con ListCreate.
   ListGoNext sposta il puntatore Current all'elemento seguente, se esiste,
   altrimenti non fa niente. Se l'elemento seguente e' in realta' il dummy,
   pone Current a NULL (per funzionare insieme a ListHasEnded).
   ListGoHead sposta il puntatore Current al primo elemento della lista, a
   meno che essa non contenga solo il dummy, nel qual caso lo pone a NULL.
   ListReadCurrent restituisce il valore dell'elemento corrente, o il valore
   d'errore se esso e' il dummy element o la lista non esiste.
   ListWriteCurrent modifica il valore dell'elemento corrente.
   ListPutFront inserisce un elemento, avente il valore indicato, in cima
   alla lista; restituisce 1 se ok, 0 se errore (manca memoria...).
   ListGetFront elimina un elemento dalla cima della lista, restituendone il
   valore. Restituisce il valore d'errore se non ci sono piu' elementi.
   ListIsEmpty restituisce 1 (vero) se la lista e' vuota o non esiste, 0
   altrimenti.
   ListHasEnded restituisce 1 (vero) se il puntatore corrente punta al
   dummy element., altrimenti 0; nel caso 1, riporta il puntatore corrente
   a inizio lista. Pertanto il ciclo corretto per percorrere la lista e':

   ListGoHead(...);
   while (!ListHasEnded(...))
      {operazioni;
       ListGoNext(...);}

   (e al suo termine Current punta a Head; il ciclo funziona, non facendo
   niente, anche a lista vuota, e persino se il descrittore della lista
   non e' stato definito!!).
   Si noti che in questa implementazione il puntatore Current puo' (a
   differenza di quanto visto a lezione) assumere il valore NULL non solo
   se la lista e' vuota, ma anche a lista piena, per indicare che si sta
   puntando al dummy element (ossia, come risultato di una chiamata a
   ListGoNext partendo dall'ultimo elemento). Questo semplifica il
   percorrimento mediante ciclo while della lista; comunque, tutte le
   chiamate a procedure del tipo ReadCurrent... quando il Current e' NULL
   restituiranno il valore d'errore. E' buona norma comunque riportare
   subito Current a un valore valido, come fa automaticamente ListHasEnded.

   Il valore d'errore (LIST_VAL_ERR) e' definito nell'header, ed e' anche il
   valore assunto dal dummy element; se esiste un valore che non puo' mai
   essere assunto dai dati validi, e' bene porre LIST_VAL_ERR uguale ad esso,
   in modo da accorgersi degli errori.
*/

#endif