
/* STRINLIB 1.6 (C) 1997 Vittorio Bertola - 21/4/97 */


/* Ricorda anche le funzioni di string.h, come strstr, strupr e strlwr! */


#ifndef _STRINLIB

#define _STRINLIB


#include <string.h>
#include <ctype.h>
/* La riga seguente deve essere "commentata via" se si compila sotto Dos
   con Turbo C++, deve essere invece presente se si compila sotto Unix
   con GCC. */
/* #include "string2t.h" */

#define MAX_WORDLEN 30



#define MAX_STRSETLEN 10
#define MAX_STRSET_STRLEN 100

typedef char strset_t[MAX_STRSETLEN][MAX_STRSET_STRLEN+1];
/* Il tipo strset_t corrisponde ad un insieme di stringhe, che puo' contenere
   al piu' MAX_STRSETLEN stringhe lunghe MAX_STRSET_STRLEN. */


static strset_t no_convacc_set = { "po'", "fa'" };
/* Insieme delle parole eccezioni a conversione apostrofo -> accento. */



typedef int IS_CHAR_F(char);
/* Le funzioni di tipo IS_CHAR_F valutano un carattere, restituendo 1 o
   0 a seconda che il carattere ricada o non ricada in una data categoria.
   Possono essere definite e fornite come argomento a certe funzioni. */



/* Funzioni di categoria IS_CHAR_F predefinite: */


extern int is_spacer(char);
/* E' vera se il carattere e' spazio o tab, altrimenti falsa. */


extern int isnot_spacer(char);
/* E' falsa se il carattere e' spazio o tab, altrimenti vera. */


extern int is_crlf(char);
/* E' vera se il carattere e' CR o LF, altrimenti falsa. */


extern int isnot_crlf(char);
/* E' falsa se il carattere e' CR o LF, altrimenti vera. */


extern int is_cursor(char);
/* E' vera se il carattere e' spazio, tab, CR o LF, altrimenti falsa. */


extern int isnot_cursor(char);
/* E' falsa se il carattere e' spazio, tab, CR o LF, altrimenti vera. */



/* Funzioni di conversione tra numeri e caratteri */

extern char basechar(int n, int radix);
/* Restituisce la lettera corrispondente al numero n (< radix) in base
   radix. L'alfabeto e' 0..9A..Z (indef. per radix > 36). Restituisce
   '#' in caso di errore (n < 0 o n >= radix). */



/* Funzioni di confronto tra stringhe */

extern int strlcmp(char *s1, char *s2, int nmean, int sensit);
/* Restituisce l'esito del confronto tra s1 e s2 (con i valori di strcmp),
   permettendo di scegliere che tipo di confronto effettuare grazie ai due
   parametri: se sensit vale 0 il confronto e' case insensitive, altrimenti
   e' case sensitive; se nmean vale -1 il confronto e' effettuato tra le
   due stringhe nella loro interezza, mentre in caso contrario e' effet-
   tuato solo sui primi nmean caratteri a sinistra. */


extern int in_wordset(char *s, strset_t wset);
/* Restituisce 1 se la stringa s compare nel vettore di stringhe wset,
   altrimenti 0. */



/* Funzioni di elaborazione stringhe

   In generale, le funzioni seguenti restituiscono s, oppure NULL in caso
   di errori o operazioni senza significato. */

extern char *revstr(char *s);
/* Rovescia la stringa fornita come argomento. */


extern char *mvleft(char *s, int startpos, int delta);
/* Sposta di delta caratteri a sinistra la parte di stringa a partire dalla
   posizione startpos, accorciando la stringa e sovrascrivendo la parte
   nel mezzo. Restituisce NULL in caso di errore (startpos inaccettabile,
   delta eccessivo...), altrimenti s. */


extern char *mvright(char *s, int startpos, int delta, int slen);
/* Sposta di delta caratteri a destra la parte di stringa tra la posizione
   startpos e la fine, allungando la stringa e quindi duplicando la parte
   nel mezzo. slen e' lo spazio utile nella stringa (ossia, per s devono
   essere stati allocati almeno slen+1 caratteri); se non e' sufficiente,
   la nuova stringa viene troncata a destra. Restituisce NULL in caso di
   errore (startpos inaccettabile...), altrimenti s. */


extern char *rmlchar(char *, IS_CHAR_F);
/* Generica funzione di rimozione caratteri a sinistra: rimuove dalla parte
   sinistra di s tutti i caratteri per cui e' vera la funzione fornita come
   argomento. */


extern char *rmrchar(char *, IS_CHAR_F);
/* Generica funzione di rimozione caratteri a destra: rimuove dalla parte
   destra di s tutti i caratteri per cui e' vera la funzione fornita come
   argomento. */


extern char *rmlspc(char *);
/* Rimuove dal lato sinistro spazi o tab iniziali. */


extern char *rmrcursor(char *);
/* Rimuove dal lato destro spazi, tab e/o CR/LF finali. */


extern char *rmrcrlf(char *);
/* Rimuove dal lato destro CR/LF finali. */


extern char *strsqz(char *);
/* Elimina spazi iniziali, spazi/CR/LF finali, converte tab/CR/LF interni e
   singoli in spazi, e riduce gli spazi/tab/CR/LF multipli interni a un
   solo spazio. */


extern char *strupfirst(char *);
/* Converte la stringa in modo che la prima lettera di ciascuna parola sia
   maiuscola, e il resto sia minuscolo. */


extern char *strcpyto(char endch, char *s, char *ds, int dslen);
/* Copia in ds la stringa s, finche' non trova in essa il carattere endch (che non viene copiato)
   o s non finisce. dslen e' il massimo numero di caratteri utili in ds. Restituisce un puntatore
   al primo carattere di s che non e' stato copiato, ossia endch, o NULL se la stringa e'
   finita. */


extern char *strcpywhile(IS_CHAR_F f, char *s, char *ds, int dslen);
/* Come la precedente, ma continua a copiare finche' e' vera la funzione f (applicata al carattere
   corrente della stringa s), e smette quando essa diventa falsa. */


extern char *strskipto(char endch, char *s);
/* Avanza nella stringa s fino a incontrare endch, o finche' la stringa non finisce. Restituisce
   un puntatore alla posizione di s in cui si e' fermata, ossia endch, o NULL se la stringa e'
   finita. */


extern char *strskipwhile(IS_CHAR_F f, char *s);
/* Come la precedente, ma avanza finche' e' vera la funzione f, applicata al carattere corrente
   della stringa s. */


extern char *skipspc(char *);
/* Come la precedente, ma avanza finche' incontra spazi o tab. */


extern char *wordof(char *s, char *p, char *ds, int dslen);
/* Restituisce la parola (delimitata da spazi) a cui, nella stringa s,
   appartiene il carattere puntato da p (=> *p deve appartenere a s);
   la memorizza in ds, e dslen e' il massimo numero di caratteri utili
   disponibile. Il risultato e' indefinito se *p e' spazio o simile. */


extern char *itofa(int n, char *s, int radix, int ndigits);
/* Genera la stringa che esprime il numero n in base radix, con esattamente
   ndigits cifre (aggiungendo zeri a sinistra). */


extern char *atoh(char *, int);
/* Converte la stringa da ASCII a HTML, sostituendo i caratteri estesi con
   le opportune entita'. Il valore intero e' la lunghezza utile nella
   stringa (ossia il numero di caratteri allocati per essa meno 1): difatti
   nella conversione la stringa si allunga, e la funzione evita di andare
   oltre lo spazio disponibile, eventualmente troncando a destra. */


extern char *htoa(char *);
/* Converte la stringa da HTML a ASCII, sostituendo le entita' HTML con i
   relativi caratteri estesi. */


extern char *spctound(char *);
/* Sostituisce tutti gli spazi e i tab nella stringa con underscore. */


extern char *undtospc(char *);
/* Sostituisce tutti gli underscore nella stringa con spazi. */


extern char *coupletoaccent(char *s);
/* Converte tutte le coppie vocale + apostrofo contenute in s nella corri-
   spondente lettera accentata; riconosce alcune eccezioni note in italiano
   (po', fa') (vedi inizio di questo file). */



/* Funzioni di libreria stringhe utili */

extern char *itmon(int);
/* Restituisce il nome esteso in italiano del mese fornito come argomento. */



#endif